@@ -161,11 +161,7 @@ class Scheduler:
             return 'cancelled'
         elif future.exception() is not None:
             logger.warning(f'FAILED future: {future}:')
-            import traceback
-
-            e = future.exception()
-            lines = traceback.format_exception(type(e), e, None)
-            logger.warning(''.join(lines).rstrip())
+            # raise future.exception()
             return 'failed'
         else:
             return 'success'
